% Name: pdfPsdPlot.m - a script to read through the PSD files and creates a PDF
%
% Copyright (C) 2011  Product Team, IRIS Data Management Center 
%
%    This is a free software; you can redistribute it and/or modify
%    it under the terms of the GNU Lesser General Public License as
%    published by the Free Software Foundation; either version 3 of the
%    License, or (at your option) any later version.
%    
%    This script is distributed in the hope that it will be useful, but
%    WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%    Lesser General Public License (GNU-LGPL) for more details.  The
%    GNU-LGPL and further information can be found here:
%    http://www.gnu.org/
%
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Description:
%
%    Script reads through the PSD files in a directory and creates a PDF.
%    The codes sets up a matrix, the rows are dB values, in 1 dB increments,
%    the columns are the frequency values, assuning the frequencies for all files fo
%    a given network, station, location and channel to be the same. Each matrix 
%    element is simply a tally of how many spectra touch that frequency, dB point.
%
% Input:
%
%    Script expects PSDs to be in the standard DMC PSD format. For more informatio
%    see http://www.iris.edu/dms/products/pdfpsd
%
%    DMC PDFPSD standard directory structure 
%
%    PDFPSD/network.station.location/channel/Yyyyy/Djjj.bin
%   
%    where
%
%    PDFPSD   - is the root directory of the PSD data files
%    network  - is the network code
%    station  - is the station code
%    location - is the location ID
%    channel  - is the channel code
%    yyyy     - is the year of the data file
%    jjj      - is the Julian day of the data file
%    
% History:
%
%    2014-05-15 IRIS DMC Product Team (MB): fixed introduced by 2014-05-14. Script was processing all locations
%                    regardless of the value given in the location parameter 
%    2014-05-14 IRIS DMC Product Team (MB): fixed scripts failure to process a single network (wildcard works!)
%    2011-05-26 IRIS DMC Product Team (MB): revised for public distribution
%    2011 Bob Woodward (IRIS): Original version 
%
%
clear;
%
% ==========================================
% configuration parameters. Change as needed
% ==========================================
%
%
% set the color map
%
cmap       = load('pdfpsd.cp'); % load the optional color map
%cmap       = colormap(flipud(hot)); % set the colormap to standard hot (comment out this line to use colormap above)
%
% define the following processing parameters based on the above input informtion
% to control the PDF generation
%
rootDir    = 'PDFPSD';
network    = ['US']; % network(s) to process, for multiple networks ['NET1';'NET2' ...]
station    = ['LTX']; % station(s) to process, for multiple stations ['STA1';'STA2' ...]
location   = ['--']; % location(s) to process, for multiple locations ['LOC1';'LOC2' ...]
channel    = ['BHZ']; % channel(s) to process, for multiple channels ['CHAN1';'CHAN2' ...]
year       = ['2004';'2005';'2006';'2007']; % year(s) to process, for multiple years ['YEAR1';'YEAR2' ...]
skiphigh   = 5; % skip the 5 highest frequencies - they always seem to be high in the PSDs
cumulative = 0; % 0- report results as percent; 1- report results as cumulative PSD
debug      = 1; % write debug messages
doplot     = 1; % create plot(s)
xmin       = 0.05; % x-axis minimum
xmax       = 100; % x-axis maximum 
ymin       = -200; % y-axis minimum 
ymax       = -50; % y-axis maximum
ytick      = 10; % y-axis tick spacing
%
% ==========================================
% code starts here
% ==========================================
%
%
% read the LNM
%
[lnmt lnmv] = textread('LNM.dat', ...
'%f %f');
%
%
% read the LNM
%
[hnmt hnmv] = textread('HNM.dat', ...
'%f %f');

%
% network loop
%
for inet = 1:length(network(:,1))
    net = network(inet,:);
    if(debug>0)fprintf('processing network: %s\n',net);end;
    %
    % station loop
    % 
    for ista = 1:length(station(:,1))
       sta = station(ista,:);
       if(debug>0)fprintf('processing station: %s\n',sta);end;
       %
       % station loop
       % 
       for iloc = 1:length(location(:,1))
          loc = location(iloc,:);
          if(debug>0)fprintf('processing location: %s\n',loc);end;
          %
          % 2014-05-14 MB: start at the root directory and look for the stations
          %
          path1 = sprintf('%s',strcat(rootDir,'/',net,'.*.',loc)); 
          if(debug>0)fprintf('searching directory for stations: %s\n',path1);end;
          stadirlist = dir(path1) ;
          for ilist = 1:length(stadirlist)
             stadir = stadirlist(ilist).name;
             %
             % 2014-05-14 MB: avoid the "." hidden objects
             %
             if(stadir(1) == '.'), continue, end;
             if(debug>0)fprintf('searching directory for channels: %s\n',stadir);end;
             %
             % channel loop
             % 
             for ichan = 1:length(channel(:,1))
                chan = channel(ichan,:);
                titlestring  = [stadir ' ' chan];
                filestring   = strcat(stadir,'.',chan);
                if(debug>0)fprintf('processing channel: %s\n',chan);end;
                %
                % year loop
                %
                numval = 0;
                nfile  = 0;
                for iyear = 1:length(year(:,1))
                    yyyy = year(iyear,:);
                    if(iyear==1)
                      titlestring = [titlestring ' ' yyyy];
                      filestring  = [filestring '_' yyyy];
                    elseif(iyear == length(year(:,1)))
                      titlestring = [titlestring '-' yyyy];
                      filestring  = [filestring '-' yyyy];
                    end; %title
  
                    %tic
                    if(debug>0)fprintf('\nprocessing year: %s\n',yyyy);end;
                    path = sprintf('%s/%s/%s/Y%s/*.bin',rootDir,stadir,chan,yyyy); % set up prefix
                    if(debug>0)fprintf('searching directory: %s\n',path);end;
                    %
                    % get .bin file list
                    %
                    dayfilelist = dir(path);
                    %
                    % PSD files loop
                    %
                    for k = 1 : length(dayfilelist)
                         ddd   = sscanf(dayfilelist(k).name(2:4),'%d');
                         if(debug>0)fprintf('processing day: %d\n',ddd);end;
                         psfile = sprintf('%s/%s/%s/Y%s/%s',rootDir,stadir,chan,yyyy,dayfilelist(k).name);
                         if(debug>0)fprintf('processing file: %s\n',psfile);end;
                         tmp   = importdata(psfile);
                         % 
                         % ignore empty files
                         % 
                         if isempty(tmp), continue, end;
                         %
                         % Note: there is no checking of the actual freq values -
                         % it assumes that there are the same number of frequencies
                         % present for the same station/channel/location
                         %
                         % extract unique frequency values
                         %
                         frequencylist = unique(tmp(:,1));
                         if(debug>0)fprintf('unique frequency count: %d\n',length(frequencylist));end;
                         if(nfile == 0)
                            frequencycount = length(frequencylist);
                            %
                            %    psd is set up as a matrix, the rows are dB values, in 1 dB increments,
                            %    the columns are the frequency values. Each matrix element is simply a tally of
                            %    how many spectra touch that frequency, dB point.
                            %
                            psd  = zeros(201,frequencycount);
                         elseif (frequencycount ~= length(frequencylist))
                            if(debug>0)fprintf('unusual frequency count: %d %s\n',length(frequencylist),psfile);end;
                            continue;
                         end; % frequency check
                         %
                         % loop through frequencies
                         %
                         for icol=skiphigh+1:frequencycount
                             freq = frequencylist(icol);
                             %
                             % go through frequency list and select the ones match
                             %
                             for ifreq=1:length(tmp)
                                 if(tmp(ifreq,1) == freq)
                                    irow = round (-1*tmp(ifreq,2)) + 1; % round the dB values at this freq
                                    if irow < 1 || irow > 201 fprintf('unusual irow: %d\n',irow); continue; end;
                                    %fprintf('saving %d , %d\n',irow,icol);
                                    %
                                    % add psd hits to the appropriate matrix element
                                    %
                                    psd(irow,icol) = psd(irow,icol) + tmp(ifreq,3);     
                                    numval = numval + tmp(ifreq,3);
                                end; % frequency list
                             end;% matrix population
                        end; % end of frequency check
                        nfile = nfile + 1;
                   end; % end of file loop - get next day file
                end; % year
                %
                % plot
                %
                if(numval>0 && doplot>0)
                     y=-(1:201);
                     x=frequencylist;
                     if(cumulative == 0)
                        psd = psd*100/numval ;
                     end;
                     figure;
                     surface(x,y,psd,'EdgeColor','none');
                     shading interp;
                     axis tight;
                     box on;
                     set(gca,'TickDir','out');
                     set(gca,'XScale','log');
                     set(gca,'YTick',[ymin:ytick:ymax]);
                     xlabel('Period (s)','FontSize',12);
                     ylabel('Power [10log_{10}(m^{2}/s^{4}/Hz)] [dB]','FontSize',12);
                     ylim([ymin ymax]);
                     xlim([xmin xmax]);
                     colormap(cmap); % set the color map
                     if(cumulative == 0)
                        title ([titlestring ' # ' num2str(numval) ' PSDs'],'FontWeight','bold','FontSize',10);
                     else
                        title (['Cumulative PSD ' titlestring ' # ' num2str(numval) ' PSDs'],'FontWeight','bold','FontSize',10);
                     end;
                     cb  = colorbar;
                     cbl = get(cb,'ylabel');
                     set(cbl,'String', 'Probability', 'Rotation', -90); 
                     set(cbl,'Position',get(cbl,'pos')+[0.5 0 0]); 
                     hold on;
                     %
                     % Plot Peterson's LNM, move the Z level up, so it will be on top of the surface plot
                     %
                     line(lnmt,lnmv,max(max(psd))*ones(length(lnmt)),'Color',[.5 .5 .5],'LineWidth',2);
                     text( lnmt(length(lnmt)), lnmv(length(lnmt))*1.03,max(max(psd)), 'LNM','Color',[.5 .5 .5],'LineWidth',5)
                     %
                     % Plot Peterson's HNM, move the Z level up, so it will be on top of the surface plot
                     %
                     line(hnmt,hnmv,max(max(psd))*ones(length(hnmt)),'Color',[.5 .5 .5],'LineWidth',2);
                     text( hnmt(length(hnmt)), hnmv(length(hnmt))*0.97,max(max(psd)), 'HNM','Color',[.5 .5 .5],'LineWidth',5)
                     outfile = strcat(filestring,'.png');
                     saveas(gcf,outfile);
                end; % plot
                if(debug >0) fprintf('finished channel %s\n', chan);end;
            end; % channel loop
         end; % sta list loop
         if(debug >0) fprintf('finished location %s\n', loc);end;
      end; % location loop
      if(debug >0) fprintf('finished station %s\n', sta);end;
   end; % station loop
   if(debug >0) fprintf('finished network %s\n', net);end;
end; % network loop
